<?php

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

require_once __DIR__.'/libs/AliansGoogleAuthenticator.php';
 
if (!defined('_PS_VERSION_')) {
    exit;
}
 
class Alianspay extends PaymentModule
{
 
    public function __construct()
    {
        $this->author    = 'Alians Group LLC';
        $this->name      = 'alianspay';
        $this->tab       = 'payments_gateways';
        $this->version   = '1.0.0';
        $this->bootstrap = true;
        $this->need_instance = 0;
        $this->ps_versions_compliancy = ['min' => '1.7.0', 'max' => _PS_VERSION_];
        parent::__construct();
 
        $this->displayName = $this->l('AliansPay');
        $this->description = $this->l('Orange Money, MTN Mobile Money, EU mobile, VISA an MasterCard payments for PS 1.7');
    }
 
    public function install()
    {
        return parent::install()
            && $this->registerHook('paymentOptions')
            && $this->registerHook('paymentReturn')
        ;
    }

    /**
     * This method handles the module's configuration page
     * @return string The page's HTML content 
     */
    public function getContent()
    {
        $output = '';

        // this part is executed only when the form is submitted
        if (Tools::isSubmit('submit' . $this->name)) {
            // retrieve the value set by the user
            $aliansUrl = (string) Tools::getValue('ALIANSPAY_URL');
            $aliansLogin = (string) Tools::getValue('ALIANSPAY_LOGIN');
            $aliansPassword = (string) Tools::getValue('ALIANSPAY_PASSWORD');
            $aliansGuid = (string) Tools::getValue('ALIANSPAY_GUID');
            $aliansCurrency = (string) Tools::getValue('ALIANSPAY_CURRENCY');

            // check that the value is valid
            if (
                empty($aliansUrl)
                || empty($aliansLogin)
                || empty($aliansPassword)
                || empty($aliansGuid)
                || empty($aliansCurrency)
            ) {
                // invalid value, show an error
                $output = $this->displayError($this->l('Invalid Configuration values. Please correctly fill the form'));
            } else {
                // value is ok, update it and display a confirmation message
                Configuration::updateValue('ALIANSPAY_URL', $aliansUrl);
                Configuration::updateValue('ALIANSPAY_LOGIN', $aliansLogin);
                Configuration::updateValue('ALIANSPAY_PASSWORD', $aliansPassword);
                Configuration::updateValue('ALIANSPAY_GUID', $aliansGuid);
                Configuration::updateValue('ALIANSPAY_CURRENCY', $aliansCurrency);
                $output = $this->displayConfirmation($this->l('Settings updated'));
            }
        }

        // display any message, then the form
        return $output . $this->displayForm();
    }

    /**
     * Builds the configuration form
     * @return string HTML code
     */
    public function displayForm()
    {
        // Init Fields form array
        $form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                ],
                'input' => [
                    [
                        'type' => 'text',
                        'label' => $this->l('URL'),
                        'name' => 'ALIANSPAY_URL',
                        'size' => 20,
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Login'),
                        'name' => 'ALIANSPAY_LOGIN',
                        'size' => 20,
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Password'),
                        'name' => 'ALIANSPAY_PASSWORD',
                        'size' => 20,
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Guid'),
                        'name' => 'ALIANSPAY_GUID',
                        'size' => 20,
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('currency'),
                        'name' => 'ALIANSPAY_CURRENCY',
                        'size' => 3,
                        'required' => true,
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-default pull-right',
                ],
            ],
        ];

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->table = $this->table;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&' . http_build_query(['configure' => $this->name]);
        $helper->submit_action = 'submit' . $this->name;

        // Default language
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');

        // Load current value into the form
        $helper->fields_value['ALIANSPAY_URL'] = Tools::getValue('ALIANSPAY_URL', Configuration::get('ALIANSPAY_URL'));
        $helper->fields_value['ALIANSPAY_LOGIN'] = Tools::getValue('ALIANSPAY_LOGIN', Configuration::get('ALIANSPAY_LOGIN'));
        $helper->fields_value['ALIANSPAY_PASSWORD'] = Tools::getValue('ALIANSPAY_PASSWORD', Configuration::get('ALIANSPAY_PASSWORD'));
        $helper->fields_value['ALIANSPAY_GUID'] = Tools::getValue('ALIANSPAY_GUID', Configuration::get('ALIANSPAY_GUID'));
        $helper->fields_value['ALIANSPAY_CURRENCY'] = Tools::getValue('ALIANSPAY_CURRENCY', Configuration::get('ALIANSPAY_CURRENCY'));

        return $helper->generateForm([$form]);
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $payment_options = [
            $this->getPaymentOption($params['cart']),
        ];

        return $payment_options;
    }

    public function getPaymentOption($cart)
    {
        $guid = Tools::getValue('ALIANSPAY_GUID', Configuration::get('ALIANSPAY_GUID'));
        $ga = new AliansGoogleAuthenticator();
        $pin = $ga->getCode($guid);

        $accepturl = (new Link())->getPageLink('order-confirmation') . '?id_cart='.$cart->id;//.'&id_module='.$this->module->id.'&id_order='.$this->module->currentOrder.'&key='.$customer->secure_key;

        $inputs = [
            'currency' => [
                'name' => 'currency',
                'type' => 'hidden',
                'value' => Tools::getValue('ALIANSPAY_CURRENCY', Configuration::get('ALIANSPAY_CURRENCY')),
            ],
            'amount' => [
                'name' => 'amount',
                'type' => 'hidden',
                'value' => $cart->getOrderTotal(),
            ],
            'pin' => [
                'name' => 'pin',
                'type' => 'hidden',
                'value' => $pin,
            ],
            'purchaseref' => [
                'name' => 'purchaseref',
                'type' => 'hidden',
                'value' => $cart->id,
            ],
            'description' => [
                'name' => 'description',
                'type' => 'hidden',
                'value' => $this->l('Card No. %s payment', $cart->id),
            ],
            'accepturl' => [
                'name' => 'accepturl',
                'type' => 'hidden',
                'value' => $accepturl,
            ],
            'cancelurl' => [
                'name' => 'cancelurl',
                'type' => 'hidden',
                'value' => $this->getCurrentUrl(),
            ],
            'callbackurl' => [
                'name' => 'callbackurl',
                'type' => 'hidden',
                'value' => Context::getContext()->link->getModuleLink($this->name, 'validation'),
            ],
            'userkey' => [
                'name' => 'userkey',
                'type' => 'hidden',
                'value' => Tools::getValue('ALIANSPAY_LOGIN', Configuration::get('ALIANSPAY_LOGIN')),
            ],
            'userpassword' => [
                'name' => 'userpassword',
                'type' => 'hidden',
                'value' => Tools::getValue('ALIANSPAY_PASSWORD', Configuration::get('ALIANSPAY_PASSWORD')),
            ],
        ];

        $option = new PaymentOption();
        $option->setCallToActionText($this->l('Pay via Orange Money or MTN Mobile Money'))
            ->setAction(Tools::getValue('ALIANSPAY_URL', Configuration::get('ALIANSPAY_URL')))
            ->setInputs($inputs)
            ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/momo_om.png'));

        return $option;
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    private function getCurrentUrl() 
    {
        return (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
    }
}